diff options
| author | Lorenzo Stoakes (ARM) <ljs@kernel.org> | 2026-09-23 18:17:52 +0100 |
|---|---|---|
| committer | Nathan Chancellor <nathan@kernel.org> | 2026-09-26 00:16:34 +0100 |
| commit | 83da9f3197e50d2e2c5b8b8d8358b6be6e867d7f (patch) | |
| tree | 646535faa8957f0a21d1509a4d4ae07ec3c63a27 /tools/lib/python/kdoc/parse_data_structs.py | |
| parent | 6f014bf46faf1645b6336f47390167de5c88e9d1 (diff) | |
| download | linux-next-83da9f3197e50d2e2c5b8b8d8358b6be6e867d7f.tar.gz linux-next-83da9f3197e50d2e2c5b8b8d8358b6be6e867d7f.zip | |
kallsyms: output binary data to speed output and kallsyms assembly
kallsyms generates an assembly file that consists mostly of .byte entries
containing compressed names, token strings and name-sorted sequence
numbers.
For an x86-64 build with 158k symbols that is a 37 MiB .S file which takes
0.57s to assemble each of the two to three times it is built over a kernel
build.
Each time it is generated it also takes kallsyms a similar amount of time
to output it.
Avoid this overhead by instead outputting this data as binary and importing
it into the assembly using the .incbin directive.
Tables that are wider than a byte remain part of the assembly to ensure
endianness and relative relocations are performed correctly.
With this change, the output assembly file shrinks from 37 MiB to 9.8 MiB,
with a 2.6 MiB binary data file alongside it, and the object remains
identical.
The generated binary file is deleted correctly on build clean along with
all other ephemeral data.
On an x86-64 system with CONFIG_KALLSYMS_ALL set:
before after delta
scripts/kallsyms 0.24s 0.18s 0.06s
assemble 0.57s 0.16s 0.41s
Per kallsyms invocation/assembly, for a total of 0.47s time saving upon
invocation.
An incremental build on the same system was reduced from 11.15s to 9.65s,
indicating a total of 1.5 seconds saved over the build.
The kallsyms runs and their assembly are on the serial tail of every build
that links vmlinux, no-op builds are unchanged.
Whole build, 128-thread Threadripper 9980X, best of N runs:
before after delta
-------------------------------
x86 defconfig, touch mm/vma.c, gcc 10.6s 9.7s -0.89s (-8%)
x86 defconfig, touch mm/vma.c, clang 10.5s 9.3s -1.2s (-11%)
x86 defconfig, clean, gcc 31.3s 30.3s -1.0s (-3%)
x86 defconfig, clean, clang 33.3s 31.7s -1.6s (-5%)
x86 allmodconfig, touch mm/vma.c, gcc 38.7s 37.2s -1.4s (-4%)
x86 allmodconfig, touch mm/vma.c, clang 39.1s 36.4s -2.6s (-7%)
Assisted-by: LLM
Reviewed-by: Kees Cook <kees@kernel.org>
Signed-off-by: Lorenzo Stoakes (ARM) <ljs@kernel.org>
Link: https://patch.msgid.link/20260923-build-speedup-v4-3-73128809a4a4@kernel.org
Signed-off-by: Nathan Chancellor <nathan@kernel.org>
Diffstat (limited to 'tools/lib/python/kdoc/parse_data_structs.py')
0 files changed, 0 insertions, 0 deletions
